from qtpy import QtWidgets, QtCore, QtGui


class Switch(QtWidgets.QWidget):
    stateChanged = QtCore.Signal(bool)

    def __init__(self, left_text, right_text, start_state=False):
        super().__init__()
        self.left_text = left_text
        self.right_text = right_text
        self.state = start_state
        self.setMinimumHeight(20)
        self.setMaximumHeight(30)
        self.setStyleSheet("border: 1px solid red;")
        self._height_perc = 60
        self.margin = 10
        self.max_center_width = 2

    def setState(self, state):
        if state != self.state:
            self.stateChanged.emit(state)
        self.state = state
        self.repaint()

    def mouseReleaseEvent(self, event: QtGui.QMouseEvent) -> None:
        self.state = not self.state
        self.stateChanged.emit(self.state)
        self.repaint()
        return super().mouseReleaseEvent(event)

    def paintEvent(self, event: QtGui.QPaintEvent) -> None:
        super().paintEvent(event)
        painter = QtGui.QPainter()
        painter.begin(self)
        painter.setRenderHint(QtGui.QPainter.Antialiasing)
        rect = self.rect()
        # dark = palette.color(QtGui.QPalette.Dark)
        # base = palette.color(QtGui.QPalette.Base)
        palette = self.palette()
        base = palette.base().color()
        text = palette.text().color()
        dot = palette.highlight().color()

        metrics = QtGui.QFontMetrics(self.font())
        left_size = metrics.size(QtCore.Qt.TextFlag.TextSingleLine, self.left_text)
        right_size = metrics.size(QtCore.Qt.TextFlag.TextSingleLine, self.right_text)
        left_size.setWidth(left_size.width() + self.margin)
        right_size.setWidth(right_size.width() + self.margin)
        # painter.drawText(0, int(rect.height() / 2 - left_size.height() / 2), self.left_text)
        painter.setPen(text)
        painter.drawText(
            0,
            0,
            left_size.width(),
            rect.height(),
            QtCore.Qt.AlignmentFlag.AlignCenter,
            self.left_text,
        )

        painter.setBrush(base)
        painter.setPen(base)
        m = int(rect.height() * ((1 - (self._height_perc / 100))) / 2)
        h = int((rect.height() - 2 * m) / 2)
        available_width = min(
            rect.width() - left_size.width() - right_size.width(),
            (self.max_center_width * self.height()),
        )
        painter.drawRoundedRect(
            left_size.width() + 1,
            m,
            available_width - 2,
            rect.height() - 2 * m,
            h,
            h,
        )
        painter.setBrush(dot)
        painter.setPen(dot)
        if self.state:
            lpos = left_size.width()
        else:
            lpos = left_size.width() + available_width - (rect.height() - 1)
        painter.drawEllipse(
            lpos,
            1,
            rect.height() - 2,
            rect.height() - 2,
        )
        painter.setPen(text)
        painter.drawText(
            left_size.width() + available_width,
            0,
            right_size.width(),
            rect.height(),
            QtCore.Qt.AlignmentFlag.AlignCenter,
            self.right_text,
        )
        painter.end()
